#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2023/11/20 15:10
# @Author  : streamer
# @File    : __init__.py.py
# @Project : celery_spider
# @Software: PyCharm
# @History : 
# VERSION     USER      DATE         DESC
# v1.0.0      Streamer   2023/11/20   CREATE
from .easy_item import BaseItem
from .easy_obj import EasyObj